#include "button.hpp"

Button::Button(DigitalInputCard *inputCard, uint8_t inputPin)
{
    this->inputCard = inputCard;
    this->inputPin = inputPin;
    this->input_handler_id = -1; // initialize to MAX_UINT16
    this->shortPressTime = 100;  // default short press time
    this->longPressTime = 2000;  // default long press time
}

Button::~Button()
{
    if (this->input_handler_id != -1)
        this->inputCard->unregisterCallback(this->input_handler_id);
}

void Button::begin()
{
    auto bindedCallback = std::bind(&Button::buttonChangedCallback, this, std::placeholders::_1, std::placeholders::_2);
    this->input_handler_id = this->inputCard->registerCallback(bindedCallback);
    // Disable Hardware Debounce
    this->inputCard->setDebounceTime(this->inputPin, 0);
}

void Button::setMinLongPressTime(uint32_t time)
{
    this->longPressTime = time;
}

void Button::setMinShortPressTime(uint32_t time)
{
    this->shortPressTime = time;
}

void Button::setLongPressCallback(std::function<void()> callback)
{
    this->longPressCallback = callback;
}

void Button::setShortPressCallback(std::function<void()> callback)
{
    this->shortPressCallback = callback;
}

void Button::buttonChangedCallback(uint8_t pin, uint8_t value)
{
    if (pin != this->inputPin)
        return;

    if (value) {
        // button pressed
        this->buttonState = true;
        this->pressedTime = millis();
    } else {
        // button released
        // If long press is not triggered, then it is a short press
        if (this->longPressTriggered == false) {
            // check if it is a short press
            if (millis() - this->pressedTime > this->shortPressTime) {
                if (this->shortPressCallback)
                    this->shortPressCallback();
            }
        }
        this->buttonState = false;
        this->longPressTriggered = false;
    }
}

void Button::loop()
{
    if (this->buttonState) {
        if (millis() - this->pressedTime > this->longPressTime && !this->longPressTriggered) {
            if (this->longPressCallback) {
                this->longPressCallback();
                this->longPressTriggered = true;
            }
        }
    }
}
